@@ -12,7 +12,7 @@ use std::iter::FromIterator;
 use std::path::{PathBuf,Path};
 use std::process::Command;
 use crate::error::RadError;
-use crate::models::RadResult;
+use crate::models::{RadResult, FlowControl};
 use crate::arg_parser::{ArgParser, GreedyState};
 use regex::Regex;
 use crate::utils::Utils;
@@ -84,6 +84,8 @@ impl BasicMacroMap {
             ("comp".to_owned(),    BMacroSign::new("comp",    ["a_content"],Self::compress)),
             ("env".to_owned(),     BMacroSign::new("env",     ["a_env_name"],Self::get_env)),
             ("envset".to_owned(),  BMacroSign::new("envset",  ["a_env_name","a_env_value"],Self::set_env)),
+            ("escape".to_owned(),  BMacroSign::new("escape",  ESR,Self::escape)),
+            ("exit".to_owned(),    BMacroSign::new("exit",    ESR,Self::exit)),
             ("fileout".to_owned(), BMacroSign::new("fileout", ["a_truncate?","a_filename","a_content"],Self::file_out)),
             ("include".to_owned(), BMacroSign::new("include", ["a_filename"],Self::include)),
             ("len".to_owned(),     BMacroSign::new("len",     ["a_string"],Self::len)),
@@ -91,6 +93,7 @@ impl BasicMacroMap {
             ("not".to_owned(),     BMacroSign::new("not",     ["a_boolean"],Self::not)),
             ("nl".to_owned(),      BMacroSign::new("nl",      ESR,Self::newline)),
             ("parent".to_owned(),  BMacroSign::new("parent",  ["a_path"],Self::get_parent)),
+            ("panic".to_owned(),   BMacroSign::new("panic",   ["a_msg"],Self::manual_panic)),
             ("path".to_owned(),    BMacroSign::new("path",    ["a_paths"],Self::merge_path)),
             ("pipe".to_owned(),    BMacroSign::new("pipe",    ["a_value"],Self::pipe)),
             ("read".to_owned(),    BMacroSign::new("read",    ["a_filename"],Self::read)),
@@ -105,6 +108,7 @@ impl BasicMacroMap {
             ("tempto".to_owned(),  BMacroSign::new("tempto",  ["a_filename"],Self::set_temp_target)),
             ("tr".to_owned(),      BMacroSign::new("tr",      ["a_source","a_matches","a_substitutions"],Self::translate)),
             ("trim".to_owned(),    BMacroSign::new("trim",    ["a_content"],Self::trim)),
+            ("triml".to_owned(),   BMacroSign::new("triml",   ["a_content"],Self::triml)),
             ("undef".to_owned(),   BMacroSign::new("undef",   ["a_macro_name"],Self::undefine_call)),
             // THis is simply a placeholder
             ("define".to_owned(),  BMacroSign::new("define",  ESR,Self::define_type)),
@@ -113,16 +117,8 @@ impl BasicMacroMap {
         // Optional macros
         #[cfg(feature = "csv")]
         {
-            map.insert("from".to_owned(),    BMacroSign::new("from",["a_macro_name","a_csv_value"],Self::from_data));
+            map.insert("from".to_owned(),    BMacroSign::new("from", ["a_macro_name","a_csv_value"],Self::from_data));
             map.insert("table".to_owned(),   BMacroSign::new("table",["a_table_form","a_csv_value"],Self::table));
-
-            // TODO Create this methods
-            // 1. Map csv value to hashmap
-            // map.insert("map".to_owned(),   BMacroSign::new("map",["a_map_name","a_csv_value"], Self::csv_map));
-            // 2. Map csv file to hashamp without printing
-            // map.insert("mapread".to_owned(),   BMacroSign::new("map",["a_map_name","a_csv_path"], Self::csv_map_include));
-            // 3. Get css row, column, table with simple querying (SQL like simple syntax)
-            // map.insert("query".to_owned(),   BMacroSign::new("query",["a_table_form","a_csv_value"], Self::csv_query));
         }
         #[cfg(feature = "chrono")]
         {
@@ -132,11 +128,13 @@ impl BasicMacroMap {
         #[cfg(feature = "lipsum")]
         map.insert("lipsum".to_owned(),      BMacroSign::new("lipsum",["a_word_count"],Self::lipsum_words));
         #[cfg(feature = "evalexpr")]
-        map.insert("eval".to_owned(),        BMacroSign::new("eval",["a_expression"],Self::eval));
+        map.insert("eval".to_owned(),        BMacroSign::new("eval",  ["a_expression"],Self::eval));
+        #[cfg(feature = "textwrap")]
+        map.insert("wrap".to_owned(),        BMacroSign::new("wrap",  ["a_width","a_content"],Self::wrap));
 
         #[cfg(feature = "hook")]
         {
-            map.insert("hookon".to_owned(),  BMacroSign::new("hookon",["a_macro_type","a_target_name"],Self::hook_enable));
+            map.insert("hookon".to_owned(),  BMacroSign::new("hookon", ["a_macro_type","a_target_name"],Self::hook_enable));
             map.insert("hookoff".to_owned(), BMacroSign::new("hookoff",["a_macro_type","a_target_name"],Self::hook_disable));
         }
 
@@ -284,6 +282,31 @@ impl BasicMacroMap {
         }
     }
 
+    /// Trim preceding and trailing whitespaces (' ', '\n', '\t', '\r') but for all lines
+    ///
+    /// # Usage
+    ///
+    /// $triml(\t multi
+    /// \t line
+    /// \t expression
+    /// )
+    fn triml(args: &str, greedy: bool, p: &mut Processor) -> RadResult<Option<String>> {
+        if let Some(args) = ArgParser::new().args_with_len(args, 1, greedy) {
+            let mut lines = String::new();
+            let mut iter = args[0].lines().peekable();
+            while let Some(line) = iter.next() {
+                lines.push_str(&Utils::trim(line));
+                // Append newline because String.lines() method cuts off all newlines
+                if let Some(_) = iter.peek() {
+                    lines.push_str(&p.state.newline);
+                }
+            }
+            Ok(Some(lines))
+        } else {
+            Err(RadError::InvalidArgument("Trim requires an argument".to_owned()))
+        }
+    }
+
     /// Removes duplicate newlines whithin given input
     ///
     /// # Usage
@@ -705,6 +728,23 @@ impl BasicMacroMap {
         }
     }
 
+    /// Trigger panic
+    fn manual_panic(args: &str, _: bool, _: &mut Processor) -> RadResult<Option<String>> {
+        Err(RadError::ManualPanic(args.to_string()))
+    }
+
+    /// Escape processing
+    fn escape(_: &str, _: bool, processor: &mut Processor) -> RadResult<Option<String>> {
+        processor.state.flow_control= FlowControl::Escape;
+        Ok(None)
+    }
+
+    /// Exit processing
+    fn exit(_: &str, _: bool, processor: &mut Processor) -> RadResult<Option<String>> {
+        processor.state.flow_control= FlowControl::Exit;
+        Ok(None)
+    }
+
     /// Merge multiple paths into a single path
     ///
     /// This creates platform agonistic path which can be consumed by other macros.
@@ -1068,7 +1108,7 @@ impl BasicMacroMap {
             processor.hook_map.switch_hook(hook_type, index, true)?;
             Ok(None)
         } else {
-            Err(RadError::InvalidArgument("hookon requires two argument".to_owned()))
+            Err(RadError::InvalidArgument("hookon requires two arguments".to_owned()))
         }
     }
 
@@ -1085,10 +1125,26 @@ impl BasicMacroMap {
             processor.hook_map.switch_hook(hook_type, index, false)?;
             Ok(None)
         } else {
-            Err(RadError::InvalidArgument("hookoff requires two argument".to_owned()))
+            Err(RadError::InvalidArgument("hookoff requires two arguments".to_owned()))
         }
     }
 
+    /// Wrap text
+    ///
+    /// * Usage
+    ///
+    /// $wrap(80, Content goes here)
+    #[cfg(feature = "textwrap")]
+    fn wrap(args: &str, _: bool, _: &mut Processor) -> RadResult<Option<String>> {
+        if let Some(args) = ArgParser::new().args_with_len(args, 2, true) {
+            let width = Utils::trim(&args[0]).parse::<usize>()?;
+            let content = &args[1];
+            let result = textwrap::fill(content, width);
+            Ok(Some(result))
+        } else {
+            Err(RadError::InvalidArgument("Wrap requires two arguments".to_owned()))
+        }
+    }
 }
 
 // TODO
