@@ -1,10 +1,11 @@
 use thiserror::Error;
-
 use crate::AuthType;
 
 /// R4d's error type
 #[derive(Error, Debug)]
 pub enum RadError {
+    #[error("Exited manually")]
+    Exit,
     #[error("Hook macro error \n= {0}")]
     HookMacroFail(String),
     #[error("Invalid conversion \n= {0}")]
@@ -49,6 +50,8 @@ pub enum RadError {
     StrictPanic,
     #[error("Processor panicked, exiting...")]
     Panic,
+    #[error("Panic triggered with message\n= {0}")]
+    ManualPanic(String)
 }
 
 // ==========
