@@ -91,12 +91,10 @@ pub(crate) struct MacroMap {
 }
 
 impl MacroMap {
-    /// Creates empty map without default basic macros
-    ///
-    /// Keyword macro cannot be empty
+    /// Creates empty map without default macros
     pub fn empty() -> Self {
         Self {
-            keyword: KeywordMacroMap::new(),
+            keyword: KeywordMacroMap::empty(),
             basic: BasicMacroMap::empty(),
             custom: HashMap::new(),
             local: HashMap::new(),
@@ -113,6 +111,10 @@ impl MacroMap {
         }
     }
 
+    pub fn clear_custom_macros(&mut self) {
+        self.custom.clear();
+    }
+
     /// Create a new local macro
     pub fn new_local(&mut self, level: usize,name: &str, value: &str) {
         self.local.insert(Utils::local_name(level,name), LocalMacro::new(level, name.to_owned(), value.to_owned()));
@@ -164,6 +166,12 @@ impl MacroMap {
         }
     }
 
+    pub fn undefine_custom(&mut self, name: &str) {
+        if self.custom.contains_key(name) {
+            self.custom.remove(name);
+        }
+    }
+
     pub fn rename(&mut self, name: &str, target: &str) {
         if self.basic.contains(name) {
             self.basic.rename(name, target);
@@ -205,6 +213,24 @@ impl MacroMap {
             .map(|(_,custom)| MacroSignature::from(custom));
         key_iter.chain(basic_iter).chain(custom_iter).collect()
     }
+
+    #[cfg(feature = "signature")]
+    pub fn get_default_signatures(&self) -> Vec<MacroSignature> {
+        let key_iter = self.keyword.macros
+            .iter()
+            .map(|(_,sig)| MacroSignature::from(sig));
+        let basic_iter = self.basic.macros
+            .iter()
+            .map(|(_,sig)| MacroSignature::from(sig));
+        key_iter.chain(basic_iter).collect()
+    }
+
+    #[cfg(feature = "signature")]
+    pub fn get_custom_signatures(&self) -> Vec<MacroSignature> {
+        self.custom
+            .iter()
+            .map(|(_,custom)| MacroSignature::from(custom)).collect()
+    }
 }
 
 /// Struct designed to check unbalanced parenthesis
@@ -385,8 +411,36 @@ impl DiffOption {
             "none" => Self::None,
             "all" => Self::All,
             "change" => Self::Change,
-            _ => return Err(RadError::InvalidConversion(format!("Diffoption, \"{}\" is not a vliad type", text))),
+            _ => return Err(RadError::InvalidConversion(format!("Diffoption, \"{}\" is not a valid type", text))),
         };
         Ok(var)
     }
 }
+
+/// Enum that controls processing flow
+pub enum FlowControl {
+    None,
+    Escape,
+    Exit,
+}
+
+#[cfg(feature = "signature")]
+pub enum SignatureType {
+    All,
+    Default,
+    Custom,
+}
+
+#[cfg(feature = "signature")]
+impl SignatureType {
+    pub fn from_str(text : &str) -> RadResult<Self> {
+        let variant = match text.to_lowercase().as_str() {
+            "all" => Self::All,
+            "default" => Self::Default,
+            "custom" => Self::Custom,
+            _ => return Err(RadError::InvalidConversion(format!("\"{}\" is not supported signature type", text)))
+        };
+
+        Ok(variant)
+    }
+}
